@@ -55,7 +55,7 @@
 
 #else
 
-#define COLOR_BG            color16(2, 8, 10)
+#define COLOR_BG            color16(0, 0, 0)
 
 #define color16(red, green, blue)                                             \
   (red << 11) | (green << 5) | blue                                           \
@@ -739,7 +739,7 @@ u32 gamepad_config_line_to_button[] =
 
 #endif
 
-#ifdef MAEMO
+#ifdef PND_BUILD
 
 u32 gamepad_config_line_to_button[] =
  { 0, 2, 1, 3, 8, 9, 10, 11, 6, 7, 4, 5, 12, 13, 14, 15 };
@@ -753,7 +753,7 @@ static const char *scale_options[] =
 #elif defined(WIZ_BUILD)
   "unscaled 3:2", "scaled 3:2 (slower)",
   "unscaled 3:2 (anti-tear)", "scaled 3:2 (anti-tear)"
-#elif defined(MAEMO)
+#elif defined(PND_BUILD)
   "unscaled", "2x", "3x", "fullscreen"
 #elif defined(GP2X_BUILD)
   "unscaled 3:2", "scaled 3:2", "fullscreen", "scaled 3:2 (software)"
@@ -762,6 +762,11 @@ static const char *scale_options[] =
 #endif
 };
 
+const char *filter2_options[] =
+{
+  "none", "scale2x", "scale3x", "eagle2x"
+};
+
 s32 load_game_config_file()
 {
   char game_config_filename[512];
@@ -808,7 +813,7 @@ s32 load_game_config_file()
 
       for(i = 0; i < 10; i++)
       {
-        cheats[i].cheat_active = file_options[3 + i] % 2;
+        cheats[i].cheat_active = file_options[4 + i] % 2;
         cheats[i].cheat_name[0] = 0;
       }
 
@@ -837,6 +842,8 @@ s32 load_game_config_file()
   return -1;
 }
 
+#define FILE_OPTION_COUNT 24
+
 s32 load_config_file()
 {
   char config_path[512];
@@ -850,7 +857,7 @@ s32 load_config_file()
     u32 file_size = file_length(config_path, config_file);
 
     // Sanity check: File size must be the right size
-    if(file_size == 92)
+    if(file_size == FILE_OPTION_COUNT * 4)
     {
       u32 file_options[file_size / 4];
       file_read_array(config_file, file_options);
@@ -859,6 +866,8 @@ s32 load_config_file()
         (sizeof(scale_options) / sizeof(scale_options[0]));
       screen_filter = file_options[1] % 2;
       global_enable_audio = file_options[2] % 2;
+      screen_filter2 = file_options[23] %
+        (sizeof(filter2_options) / sizeof(filter2_options[0]));
 
 #ifdef PSP_BUILD
       audio_buffer_size_number = file_options[3] % 10;
@@ -951,7 +960,7 @@ s32 save_config_file()
 
   if(file_check_valid(config_file))
   {
-    u32 file_options[23];
+    u32 file_options[FILE_OPTION_COUNT];
 
     file_options[0] = screen_scale;
     file_options[1] = screen_filter;
@@ -960,6 +969,7 @@ s32 save_config_file()
     file_options[4] = update_backup_flag;
     file_options[5] = global_enable_analog;
     file_options[6] = analog_sensitivity_level;
+    file_options[23] = screen_filter2;
 
 #ifndef PC_BUILD
     u32 i;
@@ -1234,11 +1244,11 @@ u32 menu(u16 *original_screen)
 
   void menu_fix_gamepad_help()
   {
-#ifndef MAEMO
+#ifndef PC_BUILD
     clear_help();
     current_option->help_string =
      gamepad_help[gamepad_config_map[
-     gamepad_config_line_to_button[number]]];
+     gamepad_config_line_to_button[current_option_num]]];
 #endif
   }
 
@@ -1326,6 +1336,12 @@ u32 menu(u16 *original_screen)
      "scaling the screen. Selecting this will produce a more even and\n"
      "smooth image, at the cost of being blurry and having less vibrant\n"
      "colors.", 3),
+#endif
+#ifdef PND_BUILD
+    string_selection_option(NULL, "Scaling filter", filter2_options,
+     (u32 *)(&screen_filter2),
+     sizeof(filter2_options) / sizeof(filter2_options[0]),
+     "Optional pixel art scaling filter", 4),
 #endif
     string_selection_option(NULL, "Frameskip type", frameskip_options,
      (u32 *)(&current_frameskip_type), 3,
@@ -1373,7 +1389,8 @@ u32 menu(u16 *original_screen)
      "This option requires gpSP to be restarted before it will take effect.",
 #else
      "Set the size (in bytes) of the audio buffer.\n"
-     "This option requires gpSP restart to take effect.",
+     "This option requires gpSP restart to take effect.\n"
+     "Settable values may be limited by SDL implementation.",
 #endif
      10),
     submenu_option(NULL, "Back", "Return to the main menu.", 12)
@@ -1485,7 +1502,7 @@ u32 menu(u16 *original_screen)
 
 #endif
 
-#if defined(GP2X_BUILD) || defined(MAEMO)
+#if defined(GP2X_BUILD) || defined(PND_BUILD)
 
   menu_option_type gamepad_config_options[] =
   {
@@ -1505,10 +1522,10 @@ u32 menu(u16 *original_screen)
     gamepad_config_option("Start        ", 10),
 #endif
     gamepad_config_option("Select       ", 11),
-#if !defined(WIZ_BUILD) && !defined(MAEMO)
+#if !defined(WIZ_BUILD) && !defined(PND_BUILD)
     gamepad_config_option("Stick Push   ", 12),
 #endif
-#ifdef MAEMO
+#ifdef PND_BUILD
     gamepad_config_option("1            ", 12),
     gamepad_config_option("2            ", 13),
     gamepad_config_option("3            ", 14),
