@@ -10,6 +10,15 @@ static int screen_h = 480;
 static int vmode_w = 240;
 static int vmode_h = 160;
 
+// Video filtering
+static int bounce_buf[400 * 272 * 2 / 4];
+static enum software_filter {
+  SWFILTER_NONE = 0,
+  SWFILTER_SCALE2X,
+  SWFILTER_SCALE3X,
+  SWFILTER_EAGLE2X,
+} sw_filter;
+
 // Key bindings file
 static char *keys_path = NULL;
 
@@ -23,9 +32,6 @@ static GtkWidget *vout_widget;
 static HildonAnimationActor *vout_actor;
 static GtkWidget *window;
 
-// Video output buffer
-void *pl_vout_buf;
-
 // Pause pop-up
 static GtkDialog *pause_dialog = NULL;
 static int autopause = FALSE;
@@ -296,21 +302,21 @@ keybd_handler(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 
   key = 1 << key;
 
-  switch(key) {
-  case GKEY_LEFT_UP:
-    key = GKEY_LEFT  | GKEY_UP;   break;
-  case GKEY_RIGHT_UP:
-    key = GKEY_RIGHT | GKEY_UP;   break;
-  case GKEY_LEFT_DOWN:
-    key = GKEY_LEFT  | GKEY_DOWN; break;
-  case GKEY_RIGHT_DOWN:
-    key = GKEY_RIGHT | GKEY_DOWN; break;
-  case GKEY_QUIT:
-    quit(); break;
-  case GKEY_PAUSE:
-    pause_key_down = TRUE;
-    activate_pause(FALSE);
-    break;
+  switch (key) {
+    case GKEY_LEFT_UP:
+      key = GKEY_LEFT  | GKEY_UP;   break;
+    case GKEY_RIGHT_UP:
+      key = GKEY_RIGHT | GKEY_UP;   break;
+    case GKEY_LEFT_DOWN:
+      key = GKEY_LEFT  | GKEY_DOWN; break;
+    case GKEY_RIGHT_DOWN:
+      key = GKEY_RIGHT | GKEY_DOWN; break;
+    case GKEY_QUIT:
+      quit(); break;
+    case GKEY_PAUSE:
+      pause_key_down = TRUE;
+      activate_pause(FALSE);
+      break;
   }
 
   if (event->type == GDK_KEY_PRESS)
@@ -371,7 +377,7 @@ configure_handler(GtkWidget *widget, GdkEventConfigure *event, gpointer user_dat
     hildon_animation_actor_set_show (keypad_actor, TRUE);
   }
 
-  fb_set_mode(0, 0, 0, screen_scale, 0, 0);
+  fb_set_mode(0, 0, 0, screen_scale, 0, screen_filter2);
 }
 
 static void
@@ -390,6 +396,7 @@ void print_help()
          "  --accelerometer     - enable accelerometer input"                       "\n"
          "  --autopause         - pause emulation when losing focus"                "\n"
          "  --scale N           - none (0), double (1), triple (2), fullscreen (3)" "\n"
+         "  --filter N          - none (0), scale2x (1), scale3x (2), eagle2x (3)"  "\n"
          "  --frameskip-type N  - disabled (0), auto (1), manual (2)"               "\n"
          "  --frameskip-value N - how many frames should be skipped"                "\n"
          "  --random-frameskip  - some randomness in choosing which frames to skip" "\n"
@@ -435,6 +442,7 @@ int gpsp_plat_args(int *argc, char *argv[])
     else if (!strcmp(argv[i], "--accelerometer"))    accel_enable = TRUE;
     else if (!strcmp(argv[i], "--autopause"))        autopause = TRUE;
     else if (!strcmp(argv[i], "--scale"))            setiopt(screen_scale);
+    else if (!strcmp(argv[i], "--filter"))           setiopt(screen_filter2);
     else if (!strcmp(argv[i], "--frameskip-type"))   setiopt(current_frameskip_type);
     else if (!strcmp(argv[i], "--frameskip-value"))  setiopt(frameskip_value);
     else if (!strcmp(argv[i], "--random-frameskip")) random_skip = TRUE;
@@ -688,6 +696,21 @@ void *fb_flip_screen(void)
       accel_keystate |= GKEY_DOWN;
   }
 
+  switch (sw_filter) {
+    case SWFILTER_SCALE2X:
+      neon_scale2x_16_16(bounce_buf, vout_image->mem, vmode_w, vmode_w*2, vmode_w*2*2, vmode_h);
+      break;
+    case SWFILTER_SCALE3X:
+      neon_scale3x_16_16(bounce_buf, vout_image->mem, vmode_w, vmode_w*2, vmode_w*3*2, vmode_h);
+      break;
+    case SWFILTER_EAGLE2X:
+      neon_eagle2x_16_16(bounce_buf, vout_image->mem, vmode_w, vmode_w*2, vmode_w*2*2, vmode_h);
+      break;
+    case SWFILTER_NONE:
+    default:
+      break;
+  }
+
   // Redraw
   gtk_widget_queue_draw (vout_widget);
 
@@ -695,7 +718,7 @@ void *fb_flip_screen(void)
   while (gtk_events_pending())
     gtk_main_iteration();
 
-  return pl_vout_buf;
+  return (void *) (sw_filter ? bounce_buf : vout_image->mem);
 }
 
 void fb_wait_vsync(void)
@@ -746,6 +769,20 @@ void fb_set_mode(int w, int h, int buffers, int scale, int filter, int filter2)
     target_h = screen_h;
   }
 
+  // Decide which filter to use
+  if (filter2 == SWFILTER_SCALE3X && vmode_h*3 > screen_h)
+    filter2 = SWFILTER_SCALE2X;
+
+  if ((filter2 == SWFILTER_SCALE2X || filter2 == SWFILTER_EAGLE2X) && vmode_h*2 > screen_h)
+    filter2 = SWFILTER_NONE;
+
+  sw_filter = filter2;
+
+  // Resolution multiplier
+  int k = filter2 == SWFILTER_SCALE2X ? 2 :
+          filter2 == SWFILTER_SCALE3X ? 3 :
+          filter2 == SWFILTER_EAGLE2X ? 2 : 1;
+
   // Calculate the output position on the screen
   int target_x = (screen_w - target_w) / 2;
   int target_y = (screen_h - target_h) / 2;
@@ -755,16 +792,13 @@ void fb_set_mode(int w, int h, int buffers, int scale, int filter, int filter2)
     gdk_image_destroy(vout_image);
 
   // Create a new output image
-  vout_image = gdk_image_new (GDK_IMAGE_FASTEST, gdk_visual_get_system(), vmode_w, vmode_h);
-
-  // Expose the new output image to the emulator
-  pl_vout_buf = (void *) vout_image->mem;
+  vout_image = gdk_image_new (GDK_IMAGE_FASTEST, gdk_visual_get_system(), vmode_w*k, vmode_h*k);
 
   // Connect the output image to the output widget
   gtk_image_set_from_image (GTK_IMAGE(vout_widget), vout_image, NULL);
 
   // Apply scaling and positioning
-  gtk_window_resize (GTK_WINDOW (vout_actor), vmode_w, vmode_h);
+  gtk_window_resize (GTK_WINDOW (vout_actor), vmode_w*k, vmode_h*k);
   hildon_animation_actor_set_position (vout_actor, target_x, target_y);
-  hildon_animation_actor_set_scale (vout_actor, (gdouble)target_w/(gdouble)vmode_w, (gdouble)target_h/(gdouble)vmode_h);
+  hildon_animation_actor_set_scale (vout_actor, (gdouble)target_w/vmode_w/k, (gdouble)target_h/vmode_h/k);
 }
